/*
 * aTunes 1.12.0
 * Copyright (C) 2006-2009 Alex Aranda, Sylvain Gaudard, Thomas Beckers and contributors
 *
 * See http://www.atunes.org/wiki/index.php?title=Contributing for information about contributors
 *
 * http://www.atunes.org
 * http://sourceforge.net/projects/atunes
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

package net.sourceforge.atunes.gui.views.panels;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.JToggleButton;
import javax.swing.SwingConstants;

import net.sourceforge.atunes.gui.GuiUtils;
import net.sourceforge.atunes.gui.images.ImageLoader;
import net.sourceforge.atunes.gui.views.controls.playerControls.KaraokeButton;
import net.sourceforge.atunes.gui.views.controls.playerControls.MuteButton;
import net.sourceforge.atunes.gui.views.controls.playerControls.NextButton;
import net.sourceforge.atunes.gui.views.controls.playerControls.NormalizationButton;
import net.sourceforge.atunes.gui.views.controls.playerControls.PlayPauseButton;
import net.sourceforge.atunes.gui.views.controls.playerControls.PreviousButton;
import net.sourceforge.atunes.gui.views.controls.playerControls.RepeatButton;
import net.sourceforge.atunes.gui.views.controls.playerControls.ShuffleButton;
import net.sourceforge.atunes.gui.views.controls.playerControls.StopButton;
import net.sourceforge.atunes.utils.LanguageTool;

/**
 * The Class PlayerControlsPanel.
 */
public class PlayerControlsPanel extends JPanel {

	/** The Constant serialVersionUID. */
	private static final long serialVersionUID = -8647737014195638177L;

	/** The time. */
	protected JLabel time;

	/** The remaining time. */
	protected JLabel remainingTime;

	/** The progress bar. */
	protected JSlider progressBar;

	/** The shuffle button. */
	protected ShuffleButton shuffleButton;

	/** The repeat button. */
	protected RepeatButton repeatButton;

	/** The karaoke button. */
	protected KaraokeButton karaokeButton;

	/** The Normalization button. */
	protected NormalizationButton normalizeButton;
	
	/** The previous button. */
	protected PreviousButton previousButton;

	/** The play button. */
	protected PlayPauseButton playButton;

	/** The stop button. */
	protected StopButton stopButton;

	/** The next button. */
	protected NextButton nextButton;

	/** The volume button. */
	protected MuteButton volumeButton;

	/** The volume slider. */
	protected JSlider volumeSlider;

	/** The volume level */
	protected JLabel volumeLevel;

	/** The balance label. */
	protected JLabel balanceLabel;

	/** The balance slider. */
	protected JSlider balanceSlider;

	/** The playing. */
	boolean playing;

	/**
	 * Instantiates a new player controls panel.
	 */
	public PlayerControlsPanel() {
		super(new GridBagLayout());
		setBorder(BorderFactory.createEmptyBorder());
		addContent();
		GuiUtils.applyComponentOrientation(this);
	}

	/**
	 * Adds the content.
	 */
	protected void addContent() {
		time = new JLabel("0:00");
		time.setHorizontalAlignment(SwingConstants.CENTER);
		time.setPreferredSize(new Dimension(50, 20));

		progressBar = new JSlider();
		//progressBar.setPreferredSize(new Dimension(1,10));
		progressBar.setToolTipText(LanguageTool.getString("CLICK_TO_SEEK"));
		progressBar.setMinimum(0);
		progressBar.setValue(0);
		progressBar.setFocusable(false);
		//progressBar.setBorder(BorderFactory.createLineBorder(Color.GRAY));

		remainingTime = new JLabel("0:00");
		remainingTime.setHorizontalAlignment(SwingConstants.CENTER);
		remainingTime.setPreferredSize(new Dimension(50, 20));

		shuffleButton = new ShuffleButton();

		repeatButton = new RepeatButton();

		karaokeButton = new KaraokeButton();
		
		normalizeButton = new NormalizationButton();

		previousButton = new PreviousButton();
		playButton = new PlayPauseButton();
		stopButton = new StopButton();
		nextButton = new NextButton();

		volumeButton = new MuteButton();

		volumeSlider = new JSlider();
		volumeSlider.setOpaque(false);
		volumeSlider.setMinimum(0);
		volumeSlider.setMaximum(100);
		volumeSlider.setValue(50);
		volumeSlider.setFocusable(false);
		volumeSlider.setPreferredSize(new Dimension(80, 20));

		volumeLevel = new JLabel("00");
		volumeLevel.setHorizontalAlignment(SwingConstants.CENTER);
		volumeLevel.setPreferredSize(new Dimension(50, 20));
		volumeLevel.setFocusable(false);

		balanceLabel = new JLabel(ImageLoader.BALANCE_MED);
		balanceLabel.setVisible(false);

		balanceSlider = new JSlider();
		balanceSlider.setMinimum(-5);
		balanceSlider.setMaximum(5);
		balanceSlider.setValue(0);
		balanceSlider.setPreferredSize(new Dimension(40, 20));
		balanceSlider.setVisible(false);

		JPanel topPanel = new JPanel(new GridBagLayout());
		topPanel.setOpaque(false);
		GridBagConstraints c = new GridBagConstraints();
		topPanel.add(time, c);
		c.gridx = 1;
		c.weightx = 1;
		c.insets = new Insets(0, 5, 0, 5);
		c.fill = GridBagConstraints.HORIZONTAL;
		topPanel.add(progressBar, c);
		c.gridx = 2;
		c.weightx = 0;
		c.insets = new Insets(0, 0, 0, 0);
		c.fill = GridBagConstraints.NONE;
		topPanel.add(remainingTime, c);

		JPanel bottomPanel = new JPanel(new GridBagLayout());
		bottomPanel.setOpaque(false);
		c.gridx = 0;
		c.gridy = 0;
		c.weightx = 0;
		c.fill = GridBagConstraints.NONE;
		c.insets = new Insets(0, 0, 0, 0);
		setButton(bottomPanel, stopButton, c);
		c.gridx = 1;
		c.insets = new Insets(0, 2, 0, 0);
		setButton(bottomPanel, previousButton, c);
		c.gridx = 2;
		setButton(bottomPanel, playButton, c);
		c.gridx = 3;
		setButton(bottomPanel, nextButton, c);
		c.gridx = 4;
		c.insets = new Insets(0, 2, 0, 0);
		bottomPanel.add(volumeButton, c);
		c.gridx = 5;
		c.weightx = 0;
		c.fill = GridBagConstraints.NONE;
		c.insets = new Insets(0, 3, 3, 0);
		bottomPanel.add(volumeSlider, c);
		c.gridx = 6;
		c.fill = GridBagConstraints.NONE;
		c.insets = new Insets(-1, -13, 0, 0);
		bottomPanel.add(volumeLevel, c);
		c.gridx = 7;
		c.weightx = 0;
		c.fill = GridBagConstraints.NONE;
		c.insets = new Insets(0, 10, 0, 0);
		bottomPanel.add(balanceLabel, c);
		c.gridx = 8;
		c.weightx = 0;
		c.fill = GridBagConstraints.NONE;
		c.insets = new Insets(0, 3, 0, 0);
		bottomPanel.add(balanceSlider, c);
		c.gridx = 9;
		c.weightx = 1;
		c.fill = GridBagConstraints.NONE;
		c.insets = new Insets(0, 0, 0, 0);
		c.anchor = GuiUtils.getComponentOrientation().isLeftToRight() ? GridBagConstraints.EAST : GridBagConstraints.WEST;
		bottomPanel.add(shuffleButton, c);
		c.gridx = 10;
		c.weightx = 0;
		c.insets = new Insets(0, 5, 0, 0);
		bottomPanel.add(repeatButton, c);
		c.gridx = 11;
		c.weightx = 0;
		c.insets = new Insets(0, 5, 0, 0);
		bottomPanel.add(karaokeButton, c);
		//begin pata
		c.gridx = 12;
		c.weightx = 0;
		c.insets = new Insets(0, 5, 0, 0);
		bottomPanel.add(normalizeButton, c);
		//end pata
		c.gridx = 0;
		c.gridheight = 1;
		c.gridy = 0;
		c.weightx = 1;
		c.weighty = 0.5;
		c.fill = GridBagConstraints.BOTH;
		c.insets = new Insets(10, 5, 5, 5);
		add(topPanel, c);

		c.gridx = 0;
		c.gridy = 1;
		c.weightx = 1;
		c.weighty = 0.5;
		c.fill = GridBagConstraints.BOTH;
		c.insets = new Insets(2, 10, 5, 5);
		add(bottomPanel, c);
	}

	/**
	 * Gets the balance label.
	 * 
	 * @return the balance label
	 */
	public JLabel getBalanceLabel() {
		return balanceLabel;
	}

	/**
	 * Gets the balance slider.
	 * 
	 * @return the balance slider
	 */
	public JSlider getBalanceSlider() {
		return balanceSlider;
	}

	/**
	 * Gets the next button.
	 * 
	 * @return the next button
	 */
	public JButton getNextButton() {
		return nextButton;
	}

	/**
	 * Gets the play button.
	 * 
	 * @return the play button
	 */
	public JButton getPlayButton() {
		return playButton;
	}

	/**
	 * Gets the previous button.
	 * 
	 * @return the previous button
	 */
	public JButton getPreviousButton() {
		return previousButton;
	}

	/**
	 * Gets the progress bar.
	 * 
	 * @return the progress bar
	 */
	public JSlider getProgressBar() {
		return progressBar;
	}

	/**
	 * Gets the remaining time.
	 * 
	 * @return the remaining time
	 */
	public JLabel getRemainingTime() {
		return remainingTime;
	}

	/**
	 * Gets the stop button.
	 * 
	 * @return the stop button
	 */
	public JButton getStopButton() {
		return stopButton;
	}

	/**
	 * Gets the time.
	 * 
	 * @return the time
	 */
	public JLabel getTime() {
		return time;
	}

	/**
	 * Gets the volume button.
	 * 
	 * @return the volume button
	 */
	public JToggleButton getVolumeButton() {
		return volumeButton;
	}

	/**
	 * Gets the volume slider.
	 * 
	 * @return the volume slider
	 */
	public JSlider getVolumeSlider() {
		return volumeSlider;
	}

	/**
	 * Gets the volume level.
	 * 
	 * @return the volume level
	 */
	public JLabel getVolumeLevel() {
		return volumeLevel;
	}

	public NormalizationButton getNormalizeButton() {
		return normalizeButton;
	}
	
	/**
	 * Checks if is playing.
	 * 
	 * @return true, if is playing
	 */
	public boolean isPlaying() {
		return playing;
	}

	/**
	 * Sets the button.
	 * 
	 * @param panel
	 *            the panel
	 * @param b
	 *            the b
	 * @param c
	 *            the c
	 */
	protected void setButton(JPanel panel, JComponent b, GridBagConstraints c) {
		panel.add(b, c);
	}

	/**
	 * Sets the playing.
	 * 
	 * @param playing
	 *            the new playing
	 */
	public void setPlaying(boolean playing) {
		this.playing = playing;
		playButton.setPlaying(playing);
	}
	
	/**
	 * Sets the ticks painting.
	 * 
	 * @param showTicks
	 *            the new showTicks
	 */
	public void setShowTicksAndLabels(boolean showTicks) {
		getProgressBar().setPaintLabels(showTicks);
		getProgressBar().setPaintTicks(showTicks);
	}
}
