/*
 * aTunes 1.12.0
 * Copyright (C) 2006-2009 Alex Aranda, Sylvain Gaudard, Thomas Beckers and contributors
 *
 * See http://www.atunes.org/wiki/index.php?title=Contributing for information about contributors
 *
 * http://www.atunes.org
 * http://sourceforge.net/projects/atunes
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

package net.sourceforge.atunes.kernel.actions;

import java.awt.event.ActionEvent;
import java.util.ArrayList;
import java.util.List;

import javax.swing.AbstractAction;
import javax.swing.JTree;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreePath;

import net.sourceforge.atunes.gui.images.ImageLoader;
import net.sourceforge.atunes.gui.model.NavigationTableModel;
import net.sourceforge.atunes.kernel.ControllerProxy;
import net.sourceforge.atunes.kernel.handlers.PlayListHandler;
import net.sourceforge.atunes.kernel.modules.navigator.FavoritesNavigationView;
import net.sourceforge.atunes.kernel.modules.navigator.NavigationViews;
import net.sourceforge.atunes.model.AudioObject;
import net.sourceforge.atunes.utils.LanguageTool;

public class AddToPlayListAction extends AbstractAction {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1625697867534974341L;

	AddToPlayListAction() {
		super(LanguageTool.getString("ADD_TO_PLAYLIST"), ImageLoader.ADD);
		putValue(SHORT_DESCRIPTION, LanguageTool.getString("ADD_TO_PLAYLIST"));
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		if (ControllerProxy.getInstance().getNavigationController().getPopupMenuCaller() == NavigationViews.getView(FavoritesNavigationView.class).getTree()) {
			TreePath[] paths = NavigationViews.getView(FavoritesNavigationView.class).getTree().getSelectionPaths();
			if (paths != null) {
				List<AudioObject> songs = new ArrayList<AudioObject>();
				songs.addAll(((NavigationTableModel) ControllerProxy.getInstance().getNavigationController().getNavigationPanel().getNavigationTable().getModel()).getSongs());
				PlayListHandler.getInstance().addToPlayList(songs);
			}
		} else if (ControllerProxy.getInstance().getNavigationController().getPopupMenuCaller() == ControllerProxy.getInstance().getNavigationController().getNavigationPanel()
				.getNavigationTable()) {
			int[] rows = ControllerProxy.getInstance().getNavigationController().getNavigationPanel().getNavigationTable().getSelectedRows();
			if (rows.length > 0) {
				List<AudioObject> songs = new ArrayList<AudioObject>();
				for (int element : rows) {
					songs.add(((NavigationTableModel) ControllerProxy.getInstance().getNavigationController().getNavigationPanel().getNavigationTable().getModel())
							.getSongAt(element));
				}
				PlayListHandler.getInstance().addToPlayList(songs);
			}
		} else {
			TreePath[] paths = ((JTree) ControllerProxy.getInstance().getNavigationController().getPopupMenuCaller()).getSelectionPaths();
			if (paths != null) {
				List<AudioObject> songs = new ArrayList<AudioObject>();
				for (TreePath element : paths) {
					songs.addAll(ControllerProxy.getInstance().getNavigationController().getAudioObjectsForTreeNode(NavigationViews.getCurrentView().getClass(),
							((DefaultMutableTreeNode) element.getLastPathComponent())));
				}
				PlayListHandler.getInstance().addToPlayList(songs);
			}
		}

	}

}
