/*
 * aTunes 1.12.0
 * Copyright (C) 2006-2009 Alex Aranda, Sylvain Gaudard, Thomas Beckers and contributors
 *
 * See http://www.atunes.org/wiki/index.php?title=Contributing for information about contributors
 *
 * http://www.atunes.org
 * http://sourceforge.net/projects/atunes
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

package net.sourceforge.atunes.kernel.actions;

import java.awt.event.ActionEvent;
import java.util.List;

import javax.swing.AbstractAction;
import javax.swing.JComponent;
import javax.swing.JTable;
import javax.swing.JTree;
import javax.swing.tree.DefaultMutableTreeNode;

import net.sourceforge.atunes.gui.images.ImageLoader;
import net.sourceforge.atunes.gui.model.NavigationTableModel;
import net.sourceforge.atunes.kernel.ControllerProxy;
import net.sourceforge.atunes.kernel.modules.navigator.NavigationViews;
import net.sourceforge.atunes.kernel.modules.repository.audio.AudioFile;
import net.sourceforge.atunes.kernel.modules.repository.tags.writer.ClearTagsProcess;
import net.sourceforge.atunes.model.AudioObject;
import net.sourceforge.atunes.model.TreeObject;
import net.sourceforge.atunes.utils.LanguageTool;

public class ClearTagAction extends AbstractAction {

	/**
	 * 
	 */
	private static final long serialVersionUID = 4476719536754930347L;

	ClearTagAction() {
		super(LanguageTool.getString("CLEAR_TAG"), ImageLoader.DELETE_TAG);
		putValue(SHORT_DESCRIPTION, LanguageTool.getString("CLEAR_TAG"));
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		JComponent caller = ControllerProxy.getInstance().getNavigationController().getPopupMenuCaller();
		List<AudioObject> files = null;
		if (caller instanceof JTable) {
			int[] rows = ((JTable) caller).getSelectedRows();
			files = ((NavigationTableModel) ((JTable) caller).getModel()).getAudioObjectsAt(rows);
		} else if (caller instanceof JTree) {
			DefaultMutableTreeNode treeNode = (DefaultMutableTreeNode) ((JTree) caller).getSelectionPath().getLastPathComponent();
			if (treeNode.getUserObject() instanceof TreeObject) {
				files = ControllerProxy.getInstance().getNavigationController().getAudioObjectsForTreeNode(NavigationViews.getCurrentView().getClass(), treeNode);
			}
		}

		if (files != null) {
			// Call process
			ClearTagsProcess process = new ClearTagsProcess(AudioFile.getAudioFiles(files));
			process.execute();
		}
	}

}
