/*
 * aTunes 1.12.0
 * Copyright (C) 2006-2009 Alex Aranda, Sylvain Gaudard, Thomas Beckers and contributors
 *
 * See http://www.atunes.org/wiki/index.php?title=Contributing for information about contributors
 *
 * http://www.atunes.org
 * http://sourceforge.net/projects/atunes
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

package net.sourceforge.atunes.kernel.actions;

import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreePath;

import net.sourceforge.atunes.gui.images.ImageLoader;
import net.sourceforge.atunes.gui.model.NavigationTableModel;
import net.sourceforge.atunes.kernel.ControllerProxy;
import net.sourceforge.atunes.kernel.handlers.FavoritesHandler;
import net.sourceforge.atunes.kernel.modules.navigator.FavoritesNavigationView;
import net.sourceforge.atunes.kernel.modules.navigator.NavigationViews;
import net.sourceforge.atunes.kernel.modules.repository.audio.AudioFile;
import net.sourceforge.atunes.model.AudioObject;
import net.sourceforge.atunes.model.TreeObject;
import net.sourceforge.atunes.utils.LanguageTool;

public class RemoveFromFavoritesAction extends AbstractAction {

	/**
	 * 
	 */
	private static final long serialVersionUID = -4288879781314486222L;

	RemoveFromFavoritesAction() {
		super(LanguageTool.getString("REMOVE_FROM_FAVORITES"), ImageLoader.DELETE_TAG);
		putValue(SHORT_DESCRIPTION, LanguageTool.getString("REMOVE_FROM_FAVORITES"));
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		if (ControllerProxy.getInstance().getNavigationController().getPopupMenuCaller() == NavigationViews.getView(FavoritesNavigationView.class).getTree()) {
			TreePath[] paths = NavigationViews.getView(FavoritesNavigationView.class).getTree().getSelectionPaths();
			if (paths != null) {
				for (TreePath element : paths) {
					TreeObject obj = (TreeObject) ((DefaultMutableTreeNode) element.getLastPathComponent()).getUserObject();
					FavoritesHandler.getInstance().removeFromFavorites(obj);
				}
			}
		} else {
			int[] rows = ControllerProxy.getInstance().getNavigationController().getNavigationPanel().getNavigationTable().getSelectedRows();
			if (rows.length > 0) {
				for (int element : rows) {
					AudioObject file = ((NavigationTableModel) ControllerProxy.getInstance().getNavigationController().getNavigationPanel().getNavigationTable().getModel())
							.getSongAt(element);
					if (file instanceof AudioFile) {
						FavoritesHandler.getInstance().removeSongFromFavorites((AudioFile) file);
					}
				}
			}
		}
		NavigationViews.refreshViews(FavoritesNavigationView.class);
	}

}
