/*
 * aTunes 1.12.0
 * Copyright (C) 2006-2009 Alex Aranda, Sylvain Gaudard, Thomas Beckers and contributors
 *
 * See http://www.atunes.org/wiki/index.php?title=Contributing for information about contributors
 *
 * http://www.atunes.org
 * http://sourceforge.net/projects/atunes
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

package net.sourceforge.atunes.kernel.actions;

import java.awt.event.ActionEvent;
import java.util.ArrayList;
import java.util.List;

import javax.swing.AbstractAction;
import javax.swing.JTree;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreePath;

import net.sourceforge.atunes.gui.images.ImageLoader;
import net.sourceforge.atunes.kernel.ControllerProxy;
import net.sourceforge.atunes.kernel.handlers.FavoritesHandler;
import net.sourceforge.atunes.kernel.modules.navigator.NavigationViews;
import net.sourceforge.atunes.kernel.modules.repository.audio.AudioFile;
import net.sourceforge.atunes.model.AudioObject;
import net.sourceforge.atunes.model.TreeObject;
import net.sourceforge.atunes.utils.LanguageTool;

public class SetFavoriteAlbumFromNavigatorAction extends AbstractAction {

	/**
	 * 
	 */
	private static final long serialVersionUID = 836910667009804037L;

	SetFavoriteAlbumFromNavigatorAction() {
		super(LanguageTool.getString("SET_FAVORITE_ALBUM"), ImageLoader.FAVORITE);
		putValue(SHORT_DESCRIPTION, LanguageTool.getString("SET_FAVORITE_ALBUM"));
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		TreePath[] paths = ((JTree) ControllerProxy.getInstance().getNavigationController().getPopupMenuCaller()).getSelectionPaths();
		if (paths != null) {
			List<AudioObject> songs = new ArrayList<AudioObject>();
			for (TreePath element : paths) {
				DefaultMutableTreeNode treeNode = (DefaultMutableTreeNode) element.getLastPathComponent();
				if (treeNode.getUserObject() instanceof TreeObject) {
					songs.addAll(ControllerProxy.getInstance().getNavigationController().getAudioObjectsForTreeNode(NavigationViews.getCurrentView().getClass(), treeNode));
				}
			}
			FavoritesHandler.getInstance().addFavoriteAlbums(AudioFile.getAudioFiles(songs));
			ControllerProxy.getInstance().getNavigationController().refreshTable();
		}
	}

}
