/*
 * aTunes 1.12.0
 * Copyright (C) 2006-2009 Alex Aranda, Sylvain Gaudard, Thomas Beckers and contributors
 *
 * See http://www.atunes.org/wiki/index.php?title=Contributing for information about contributors
 *
 * http://www.atunes.org
 * http://sourceforge.net/projects/atunes
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

package net.sourceforge.atunes.kernel.controllers.editPreferencesDialog;

import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import javax.swing.DefaultListModel;
import javax.swing.SwingConstants;

import net.sourceforge.atunes.gui.views.dialogs.editPreferences.ContextInformationPanel;
import net.sourceforge.atunes.gui.views.dialogs.editPreferences.DevicePanel;
import net.sourceforge.atunes.gui.views.dialogs.editPreferences.EditPreferencesDialog;
import net.sourceforge.atunes.gui.views.dialogs.editPreferences.GeneralPanel;
import net.sourceforge.atunes.gui.views.dialogs.editPreferences.InternetPanel;
import net.sourceforge.atunes.gui.views.dialogs.editPreferences.LastFmPanel;
import net.sourceforge.atunes.gui.views.dialogs.editPreferences.NavigatorPanel;
import net.sourceforge.atunes.gui.views.dialogs.editPreferences.OSDPanel;
import net.sourceforge.atunes.gui.views.dialogs.editPreferences.PlayListPrefPanel;
import net.sourceforge.atunes.gui.views.dialogs.editPreferences.PlayerPanel;
import net.sourceforge.atunes.gui.views.dialogs.editPreferences.PodcastFeedPanel;
import net.sourceforge.atunes.gui.views.dialogs.editPreferences.PreferencesPanel;
import net.sourceforge.atunes.gui.views.dialogs.editPreferences.RadioPanel;
import net.sourceforge.atunes.gui.views.dialogs.editPreferences.RepositoryPanel;
import net.sourceforge.atunes.kernel.ControllerProxy;
import net.sourceforge.atunes.kernel.Kernel;
import net.sourceforge.atunes.kernel.actions.Actions;
import net.sourceforge.atunes.kernel.actions.AddLovedSongInLastFMAction;
import net.sourceforge.atunes.kernel.controllers.model.DialogController;
import net.sourceforge.atunes.kernel.handlers.ContextInformationHandler;
import net.sourceforge.atunes.kernel.handlers.HotkeyHandler;
import net.sourceforge.atunes.kernel.handlers.PodcastFeedHandler;
import net.sourceforge.atunes.kernel.handlers.RadioHandler;
import net.sourceforge.atunes.kernel.handlers.SystemTrayHandler;
import net.sourceforge.atunes.kernel.handlers.VisualHandler;
import net.sourceforge.atunes.kernel.modules.contextInformation.lyrics.engines.LyricsEngineInfo;
import net.sourceforge.atunes.kernel.modules.device.DeviceConnectionMonitor;
import net.sourceforge.atunes.kernel.modules.navigator.NavigationViews;
import net.sourceforge.atunes.kernel.modules.navigator.RepositoryNavigationView;
import net.sourceforge.atunes.kernel.modules.repository.tags.tag.TagAttribute;
import net.sourceforge.atunes.kernel.modules.state.ApplicationState;
import net.sourceforge.atunes.kernel.modules.state.beans.LocaleBean;
import net.sourceforge.atunes.kernel.modules.state.beans.ProxyBean;
import net.sourceforge.atunes.utils.LanguageTool;
import net.sourceforge.atunes.utils.StringUtils;
import net.sourceforge.atunes.utils.log4j.LogCategories;

/**
 * The Class EditPreferencesDialogController.
 */
public class EditPreferencesDialogController extends DialogController<EditPreferencesDialog> {

	/** The Constant GENERAL. */
	private static final String GENERAL = LanguageTool.getString("GENERAL");

	/** The Constant REPOSITORY. */
	private static final String REPOSITORY = LanguageTool.getString("REPOSITORY");

	/** The Constant PLAYER. */
	private static final String PLAYER = LanguageTool.getString("PLAYER");

	/** The Constant NAVIGATOR. */
	private static final String NAVIGATOR = LanguageTool.getString("NAVIGATOR");

	/** The Constant OSD. */
	private static final String OSD = LanguageTool.getString("OSD");

	/** The Constant AUDIO_SCROBBLER. */
	private static final String CONTEXT_INFORMATION = LanguageTool.getString("CONTEXT_INFORMATION");

	/** The Constant INTERNET. */
	private static final String INTERNET = LanguageTool.getString("INTERNET");

	/** The Constant LASTFM. */
	private static final String LASTFM = "Last.fm";

	/** The Constant DEVICE. */
	private static final String DEVICE = LanguageTool.getString("DEVICE");

	/** The Constant RADIO. */
	private static final String RADIO = LanguageTool.getString("RADIO");

	/** The Constant PODCAST_FEED. */
	private static final String PODCAST_FEED = LanguageTool.getString("PODCAST_FEEDS");
	
	/** The Constant PLAYLIST. */
	private static final String PLAYLIST = LanguageTool.getString("PLAYLIST");

	/** The tree nodes. */
	private String[] treeNodes = new String[] { GENERAL, REPOSITORY, PLAYER, NAVIGATOR, PLAYLIST, OSD, CONTEXT_INFORMATION, INTERNET, LASTFM, DEVICE, RADIO, PODCAST_FEED };

	/** The internet panel. */
	private InternetPanel internetPanel;

	/** The state. */
	private ApplicationState state = Kernel.getInstance().state;

	/** The panels. */
	private PreferencesPanel[] panels = new PreferencesPanel[] { getGeneralPanel(), getRepositoryPanel(), getPlayerPanel(), getNavigatorPanel(), getPlayListPrefPanel(), getOSDPanel(),
			getContextInformationPanel(), getInternetPanel(), getLastFmPanel(), getDevicePanel(), getRadioPanel(), getPodcastFeedPanel()};

	/**
	 * Instantiates a new edits the preferences dialog controller.
	 */
	public EditPreferencesDialogController() {
		super(VisualHandler.getInstance().getEditPreferencesDialog());
		getDialogControlled().setPanels(panels);
		buildList();
		addBindings();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * net.sourceforge.atunes.kernel.controllers.model.Controller#addBindings()
	 */
	@Override
	protected void addBindings() {
		EditPreferencesDialogListener listener = new EditPreferencesDialogListener(getDialogControlled(), this);
		getDialogControlled().getList().addListSelectionListener(listener);
		getDialogControlled().getCancel().addActionListener(listener);
		getDialogControlled().getOk().addActionListener(listener);
		getDialogControlled().getContextInformationPanel().getClearCache().addActionListener(listener);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * net.sourceforge.atunes.kernel.controllers.model.Controller#addStateBindings
	 * ()
	 */
	@Override
	protected void addStateBindings() {
		// Nothing to do
	}

	/**
	 * Builds the list.
	 */
	private void buildList() {
		DefaultListModel listModel = new DefaultListModel();

		for (String s : treeNodes) {
			listModel.addElement(s);
		}

		getDialogControlled().setListModel(listModel);
	}

	/**
	 * Gets the audio scrobbler panel.
	 * 
	 * @return the audio scrobbler panel
	 */
	private PreferencesPanel getContextInformationPanel() {
		return new ContextInformationPanel();
	}
	
	/**
	 * Gets the device panel.
	 * 
	 * @return the device panel
	 */
	private DevicePanel getDevicePanel() {
		return new DevicePanel();
	}

	/**
	 * Gets the general panel.
	 * 
	 * @return the general panel
	 */
	private PreferencesPanel getGeneralPanel() {
		return new GeneralPanel();
	}

	/**
	 * Gets the internet panel.
	 * 
	 * @return the internet panel
	 */
	private PreferencesPanel getInternetPanel() {
		internetPanel = new InternetPanel();
		return internetPanel;
	}

	/**
	 * Gets the last fm panel.
	 * 
	 * @return the last fm panel
	 */
	private PreferencesPanel getLastFmPanel() {
		return new LastFmPanel();
	}

	/**
	 * Gets the navigator panel.
	 * 
	 * @return the navigator panel
	 */
	private PreferencesPanel getNavigatorPanel() {
		return new NavigatorPanel();
	}

	/**
	 * Gets the oSD panel.
	 * 
	 * @return the oSD panel
	 */
	private PreferencesPanel getOSDPanel() {
		return new OSDPanel();
	}

	/**
	 * Gets the player panel.
	 * 
	 * @return the player panel
	 */
	private PreferencesPanel getPlayerPanel() {
		return new PlayerPanel();
	}

	/**
	 * Gets the podcast feed panel.
	 * 
	 * @return the podcast feed panel
	 */
	private PodcastFeedPanel getPodcastFeedPanel() {
		return new PodcastFeedPanel();
	}

	/**
	 * Gets the radio panel.
	 * 
	 * @return the radio panel
	 */
	private RadioPanel getRadioPanel() {
		return new RadioPanel();
	}

	/**
	 * Gets the repository panel.
	 * 
	 * @return the repository panel
	 */
	private PreferencesPanel getRepositoryPanel() {
		return new RepositoryPanel();
	}

	
	/**
	 * Gets the playlist panel.
	 * 
	 * @return the playlist panel
	 */
	private PreferencesPanel getPlayListPrefPanel() {
		return new PlayListPrefPanel();
	}
	 
	/**
	 * No errors.
	 * 
	 * @return true, if successful
	 */
	boolean noErrors() {
		return internetPanel.getResult() != null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * net.sourceforge.atunes.kernel.controllers.model.Controller#notifyReload()
	 */
	@Override
	protected void notifyReload() {
		// Nothing to do
	}

	/**
	 * Process preferences.
	 * 
	 * @return true if application needs to be restarted to apply some changes
	 */
	boolean processPreferences() {
		boolean needRestart = false;
		// Fetch result from panels
		Map<String, Object> result = new HashMap<String, Object>();
		for (PreferencesPanel p : panels) {
			result.putAll(p.getResult());
		}

		// Set properties
		state.setShowTitle((Boolean) result.get("SHOW_TITLE"));

		boolean oldMultipleWindow = state.isMultipleWindow();
		boolean newMultipleWindow = result.get("WINDOW_TYPE").equals(LanguageTool.getString("MULTIPLE_WINDOW"));
		state.setMultipleWindow(newMultipleWindow);
		if (oldMultipleWindow != newMultipleWindow) {
			needRestart = true;
		}

		LocaleBean oldLocale = state.getLocale();
		LocaleBean newLocale = new LocaleBean((Locale) result.get("LANGUAGE"));
		state.setLocale(newLocale);
		if (!oldLocale.equals(newLocale)) {
			needRestart = true;
		}

		Boolean oldIsUseDefaultFont = state.isUseDefaultFont();
		Boolean newIsUseDefaultFont = (Boolean) result.get("USE_DEFAULT_FONT");
		state.setUseDefaultFont(newIsUseDefaultFont);
		if (!oldIsUseDefaultFont.equals(newIsUseDefaultFont)) {
			needRestart = true;
		}

		Boolean oldIsUseOSSettingsForFontSmoothing = state.isUseOSSettingsForFontSmoothing();
		Boolean newIsUseOSSettingsForFontSmoothing = (Boolean) result.get("USE_OS_SETTINGS_FOR_FONT_SMOOTHING");
		state.setUseOSSettingsForFontSmoothing(newIsUseOSSettingsForFontSmoothing);
		if (!oldIsUseOSSettingsForFontSmoothing.equals(newIsUseOSSettingsForFontSmoothing)) {
			needRestart = true;
		}

		Boolean oldIsUseFontSmoothing = state.isUseFontSmoothing();
		Boolean newUseFontSmoothing = (Boolean) result.get("USE_FONT_SMOOTHING");
		state.setUseFontSmoothing(newUseFontSmoothing);
		if (!oldIsUseFontSmoothing.equals(newUseFontSmoothing)) {
			needRestart = true;
		}

		state.setShowSystemTray((Boolean) result.get("SHOW_TRAY_ICON"));
		state.setShowTrayPlayer((Boolean) result.get("SHOW_TRAY_PLAYER"));

		String oldTheme = state.getSkin();
		String newTheme = (String) result.get("THEME");
		state.setSkin(newTheme);
		if (!oldTheme.equals(newTheme)) {
			needRestart = true;
		}

		state.setPlayAtStartup((Boolean) result.get("PLAY_AT_STARTUP"));
		state.setUseFadeAway((Boolean) result.get("USE_FADE_AWAY"));
		state.setUseShortPathNames((Boolean) result.get("USE_SHORT_PATH_NAMES_FOR_MPLAYER"));
		state.setEnableHotkeys((Boolean) result.get("ENABLE_GOLBAL_HOTKEYS"));
		state.setCacheFilesBeforePlaying((Boolean) result.get("CACHE_FILES_BEFORE_PLAYING"));
		String engine = (String) result.get("PLAYER_ENGINE");
		if (!state.getPlayerEngine().equals(engine)) {
			state.setPlayerEngine(engine);
			needRestart = true;
		}

		state.setAutoRepositoryRefreshTime((Integer) result.get("REPOSITORY_REFRESH_TIME"));
		state.setSaveRepositoryAsXml((Boolean) result.get("SAVE_REPOSITORY_AS_XML"));

		state.setShowFavoritesInNavigator((Boolean) result.get("SHOW_FAVORITES"));
		state.setShowAlbumTooltip((Boolean) result.get("SHOW_ALBUM_TOOLTIP"));
		state.setAlbumTooltipDelay((Integer) result.get("ALBUM_TOOLTIP_DELAY"));
		state.setUseSmartTagViewSorting((Boolean) result.get("USE_SMART_TAG_VIEW_SORTING"));
		state.setShowNavigatorTabsAtLeft((Boolean) result.get("SHOW_NAVIGATION_TABS_AT_LEFT"));
		state.setShowNavigatorTabsText((Boolean) result.get("SHOW_NAVIGATION_TABS_TEXT"));
		state.setHighlightIncompleteTagFolders((Boolean) result.get("HIGHLIGHT_INCOMPLETE_TAG_FOLDERS"));
		@SuppressWarnings("unchecked")
		List<TagAttribute> tagAttributesList = (List<TagAttribute>) result.get("TAG_ATTRIBUTES_FOR_FOLDERS");
		state.setHighlightIncompleteTagFoldersAttributes(tagAttributesList);
		state.setUsePersonNamesArtistTagViewSorting((Boolean) result.get("USE_PERSON_NAMES_ARTIST_TAG_VIEW_SORTING"));

		state.setAnimateOSD((Boolean) result.get("ANIMATE_OSD"));
		state.setOsdDuration((Integer) result.get("OSD_DURATION"));

		state.setUseContextInformation((Boolean) result.get("ACTIVATE_CONTEXT_INFORMATION"));
		state.setSaveContextInformationPicture((Boolean) result.get("SAVE_PICTURES_TO_AUDIO_FOLDERS"));
		state.setShowContextTabsText((Boolean) result.get("SHOW_CONTEXT_TABS_TEXT"));
		state.setHideVariousArtistsAlbums((Boolean) result.get("HIDE_VARIOUS_ARTISTS_ALBUMS"));
		state.setMinimumSongNumberPerAlbum((Integer) result.get("MINIMUM_SONG_NUMBER_PER_ALBUM"));

		state.setProxy((ProxyBean) result.get("PROXY"));

		state.setLastFmUser((String) result.get("LASTFM_USER"));
		state.setLastFmPassword((String) result.get("LASTFM_PASSWORD"));
		state.setLastFmEnabled((Boolean) result.get("LASTFM_ENABLED"));
		Actions.getAction(AddLovedSongInLastFMAction.class).setEnabled(state.isLastFmEnabled());
		@SuppressWarnings("unchecked")
		List<LyricsEngineInfo> lyricsEnginesInfo = (List<LyricsEngineInfo>) result.get("LYRICS_ENGINES_SELECTION");
		state.setLyricsEnginesInfo(lyricsEnginesInfo);

		state.setDefaultDeviceLocation((String) result.get("DEVICE_DEFAULT_LOCATION"));
		state.setFlatStructure((Boolean) result.get("FLAT"));
		state.setArtistStructure((Boolean) result.get("ARTIST"));
		state.setFullStructure((Boolean) result.get(StringUtils.getString("ARTIST", " / ", "ALBUM")));

		state.setReadInfoFromRadioStream((Boolean) result.get("READ_INFO_FROM_RADIO_STREAM"));
		state.setShowAllRadioStations((Boolean) result.get("SHOW_ALL_RADIO_STATIONS"));

		state.setPodcastFeedEntriesRetrievalInterval((Long) result.get("PODCAST_FEED_ENTRIES_RETRIEVAL_INTERVAL"));
		state.setPodcastFeedEntryDownloadPath((String) result.get("PODCAST_FEED_ENTRIES_DOWNLOAD_FOLDER"));
		state.setUseDownloadedPodcastFeedEntries((Boolean) result.get("USE_DOWNLOADED_PODCAST_FEED_ENTRIES"));
		state.setRemovePodcastFeedEntriesRemovedFromPodcastFeed((Boolean) result.get("REMOVE_PODCAST_FEED_ENTRIES_REMOVED_FROM_PODCAST_FEED"));

		state.setShowTicks((Boolean) result.get("SHOW_TICKS"));
		state.setStopPlayerOnPlayListSwitch((Boolean) result.get(PlayListPrefPanel.PROP_PLAYLIST_STOP_ON_SWITCH));
		state.setStopPlayerOnPlayListClear((Boolean) result.get(PlayListPrefPanel.PROP_PLAYLIST_STOP_ON_CLEAR));
		state.setAutoScrollPlayListEnabled((Boolean) result.get(PlayListPrefPanel.PROP_AUTO_SCROLL_PLAYLIST));
		return needRestart;
	}

	/**
	 * Start.
	 */
	public void start() {
		getLogger().debug(LogCategories.CONTROLLER);

		// Update panels
		for (PreferencesPanel panel: panels) {
			panel.updatePanel(state);
		}

		getDialogControlled().getList().setSelectedIndex(0);
		getDialogControlled().setVisible(true);
	}

	/**
	 * Update application with new configuration
	 */
	void updateApplication() {
		// Show or hide audioscrobbler panel
		VisualHandler.getInstance().showContextInformationPanel(state.isUseContextInformation(), true);

		// Set location for navigator tabs (left or top)
		VisualHandler.getInstance().getNavigationPanel().getTabbedPane().setTabPlacement(state.isShowNavigatorTabsAtLeft() ? SwingConstants.LEFT : SwingConstants.TOP);

		// Set text for navigator tabs
		VisualHandler.getInstance().getNavigationPanel().setNavigatorTabsText(state.isShowNavigatorTabsText());

		// Set text for context tabs
		VisualHandler.getInstance().getContextInformationPanel().setContextTabsText(state.isShowContextTabsText());

		// Set ticks for the player progress bar
		VisualHandler.getInstance().getPlayerControls().setShowTicksAndLabels(state.isShowTicks());

		// Once done graphic changes, repaint the window
		VisualHandler.getInstance().repaint();

		NavigationViews.refreshViews(RepositoryNavigationView.class);
		ContextInformationHandler.getInstance().updateLastFmService(state.getProxy(), Kernel.getInstance().state.getLastFmUser(), state.getLastFmPassword());
		ContextInformationHandler.getInstance().setLyricsEngines(state.getLyricsEnginesInfo());
		SystemTrayHandler.getInstance().setTrayIconVisible(state.isShowSystemTray());
		SystemTrayHandler.getInstance().setTrayPlayerVisible(state.isShowTrayPlayer());
		DeviceConnectionMonitor.startMonitor();
		PodcastFeedHandler.getInstance().setPodcastFeedEntryRetrievalInterval(state.getPodcastFeedEntriesRetrievalInterval());
		HotkeyHandler.getInstance().enableHotkeys(state.isEnableHotkeys());
		RadioHandler.getInstance().refresh();
		
		if (state.isAutoScrollPlayListEnabled()) {
			ControllerProxy.getInstance().getPlayListController().scrollPlayList(true);
		}

	}
}
