/*
 * aTunes 1.12.0
 * Copyright (C) 2006-2009 Alex Aranda, Sylvain Gaudard, Thomas Beckers and contributors
 *
 * See http://www.atunes.org/wiki/index.php?title=Contributing for information about contributors
 *
 * http://www.atunes.org
 * http://sourceforge.net/projects/atunes
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

package net.sourceforge.atunes.kernel.controllers.searchResults;

import java.util.ArrayList;
import java.util.List;

import javax.swing.table.TableRowSorter;

import net.sourceforge.atunes.gui.views.dialogs.SearchResultsDialog;
import net.sourceforge.atunes.kernel.controllers.model.DialogController;
import net.sourceforge.atunes.kernel.handlers.PlayListHandler;
import net.sourceforge.atunes.kernel.handlers.VisualHandler;
import net.sourceforge.atunes.kernel.modules.search.SearchResult;
import net.sourceforge.atunes.kernel.modules.search.SearchResultTableModel;
import net.sourceforge.atunes.kernel.modules.search.SearchableObject;
import net.sourceforge.atunes.model.AudioObject;

/**
 * Controller for SearchResultsDialog.
 */
public class SearchResultsController extends DialogController<SearchResultsDialog> {

	/** The results. */
	private List<SearchResult> results;

	/**
	 * Constructor.
	 * 
	 * @param dialog
	 *            the dialog
	 */
	public SearchResultsController(SearchResultsDialog dialog) {
		super(dialog);
		addBindings();
	}

	/**
	 * Shows dialog with search results.
	 * 
	 * @param searchableObject
	 *            the searchable object
	 * @param resultsList
	 *            the results
	 */
	public void showSearchResults(SearchableObject searchableObject, List<SearchResult> resultsList) {
		this.results = resultsList;
		SearchResultTableModel model = searchableObject.getSearchResultsTableModel(resultsList);
		getDialogControlled().getSearchResultsTable().setModel(model);
		getDialogControlled().getSearchResultsTable().setRowSorter(new TableRowSorter<SearchResultTableModel>(model));
		getDialogControlled().setVisible(true);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * net.sourceforge.atunes.kernel.controllers.model.Controller#addBindings()
	 */
	@Override
	protected void addBindings() {
		SearchResultsListener listener = new SearchResultsListener(this, getDialogControlled());
		getDialogControlled().getShowElementInfo().addActionListener(listener);
		getDialogControlled().getAddToCurrentPlayList().addActionListener(listener);
		getDialogControlled().getAddToNewPlayList().addActionListener(listener);

	}

	/**
	 * Displays info of first selected item.
	 */
	protected void showInfo() {
		List<AudioObject> selectedResults = getSelectedResults();
		if (selectedResults == null) {
			return;
		}
		VisualHandler.getInstance().showPropertiesDialog(selectedResults.get(0));
	}

	/**
	 * Adds selected results to current play list.
	 */
	protected void addToPlayList() {
		List<AudioObject> selectedResults = getSelectedResults();
		if (selectedResults == null) {
			return;
		}
		PlayListHandler.getInstance().addToPlayList(selectedResults);
	}

	/**
	 * Adds selected results to a new play list.
	 */
	protected void addToNewPlayList() {
		List<AudioObject> selectedResults = getSelectedResults();
		if (selectedResults == null) {
			return;
		}
		PlayListHandler.getInstance().newPlayList(selectedResults);
	}

	/**
	 * Returns selected results as audio objects.
	 * 
	 * @return the selected results
	 */
	private List<AudioObject> getSelectedResults() {
		int[] selectedRows = getDialogControlled().getSearchResultsTable().getSelectedRows();
		if (selectedRows.length == 0) {
			return null;
		}
		ArrayList<AudioObject> selectedResults = new ArrayList<AudioObject>();
		for (int element : selectedRows) {
			selectedResults.add(this.results.get(element).getAudioObject());
		}
		return selectedResults;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * net.sourceforge.atunes.kernel.controllers.model.Controller#addStateBindings
	 * ()
	 */
	@Override
	protected void addStateBindings() {
		// Nothing to do
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * net.sourceforge.atunes.kernel.controllers.model.Controller#notifyReload()
	 */
	@Override
	protected void notifyReload() {
		// Nothing to do
	}
}
