/*
 * aTunes 1.12.0
 * Copyright (C) 2006-2009 Alex Aranda, Sylvain Gaudard, Thomas Beckers and contributors
 *
 * See http://www.atunes.org/wiki/index.php?title=Contributing for information about contributors
 *
 * http://www.atunes.org
 * http://sourceforge.net/projects/atunes
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

package net.sourceforge.atunes.kernel.modules.contextInformation.lyrics.engines;

import java.io.IOException;
import java.net.UnknownHostException;

import net.sourceforge.atunes.kernel.modules.contextInformation.Lyrics;
import net.sourceforge.atunes.utils.StringUtils;
import net.sourceforge.atunes.utils.log4j.LogCategories;
import net.sourceforge.atunes.utils.log4j.Logger;

import org.htmlparser.Node;
import org.htmlparser.NodeFilter;
import org.htmlparser.Parser;
import org.htmlparser.tags.Div;
import org.htmlparser.util.NodeList;
import org.htmlparser.util.ParserException;

public class LyricsDirectoryEngine extends LyricsEngine {

	private static final String BASE_URL = "http://www.lyricsdir.com/";
	private static final String CHARSET = "UTF-8";

	private Logger logger = new Logger();

	@Override
	public Lyrics getLyricsFor(String artist, String title) {
		StringBuilder sb = new StringBuilder();
		sb.append(BASE_URL);
		sb.append(encodeString(getString(artist, title)));
		sb.append("lyrics");
		sb.append(".html");

		try {
			String html = readURL(getConnection(sb.toString()), CHARSET);
			Parser parser = Parser.createParser(html, CHARSET);

			// Get <div id="lyrics2"> element
			NodeList nodeList = parser.extractAllNodesThatMatch(new NodeFilter() {
				private static final long serialVersionUID = 4555419135319490609L;

				@Override
				public boolean accept(Node node) {
					return node.getText().contains("div id=\"lyrics2\"");
				}
			});

			if (nodeList.size() > 0) {
				// Remove ads
				NodeList nodeList2 = nodeList.elementAt(0).getChildren().extractAllNodesThatMatch(new NodeFilter() {
					private static final long serialVersionUID = -3109643222380150369L;

					@Override
					public boolean accept(Node node) {
						return !(node instanceof Div);
					}
				});

				String lyrics = nodeList2.asString().trim();
				return lyrics != null && !lyrics.isEmpty() ? new Lyrics(lyrics, sb.toString()) : null;
			}
			return null;
		} catch (UnknownHostException e) {
			logger.error(LogCategories.SERVICE, e);
			return null;
		} catch (IOException e) {
			logger.debug(LogCategories.SERVICE, StringUtils.getString("No lyrics found at ", BASE_URL, " for ", artist, " - ", title));
			return null;
		} catch (ParserException e) {
			logger.error(LogCategories.SERVICE, e);
			return null;
		}
	}

	private String getString(String a, String t) {
		StringBuilder sb = new StringBuilder();
		String artist = a.toLowerCase().trim();
		artist = artist.replaceAll("\\W", " ");
		String title = t.toLowerCase().trim();
		title = title.replaceAll("\\W", " ");
		for (String artistSplits : artist.split("\\s+")) {
			sb.append(artistSplits);
			sb.append("-");
		}
		for (String titleSplits : title.split(" ")) {
			sb.append(titleSplits);
			sb.append("-");
		}
		return sb.toString();
	}

	@Override
	public String getLyricsProviderName() {
		return "LyricsDirectory";
	}

	@Override
	public String getUrlForAddingNewLyrics(String artist, String title) {
		return "";
	}

}
