/*
 * aTunes 1.12.0
 * Copyright (C) 2006-2009 Alex Aranda, Sylvain Gaudard, Thomas Beckers and contributors
 *
 * See http://www.atunes.org/wiki/index.php?title=Contributing for information about contributors
 *
 * http://www.atunes.org
 * http://sourceforge.net/projects/atunes
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

package net.sourceforge.atunes.kernel.modules.device;

import java.io.File;
import java.io.IOException;
import java.util.List;

import net.sourceforge.atunes.kernel.handlers.RepositoryHandler;
import net.sourceforge.atunes.kernel.modules.process.AudioFileTransferProcess;
import net.sourceforge.atunes.kernel.modules.repository.ExportFilesProcess;
import net.sourceforge.atunes.kernel.modules.repository.audio.AudioFile;
import net.sourceforge.atunes.kernel.utils.FileNameUtils;
import net.sourceforge.atunes.kernel.utils.SystemProperties;
import net.sourceforge.atunes.utils.LanguageTool;
import net.sourceforge.atunes.utils.StringUtils;

import org.apache.commons.io.FileUtils;

/**
 * The Class TransferToRepositoryProcess.
 */
public class TransferToRepositoryProcess extends AudioFileTransferProcess {

	/** The structure. */
	private int structure;

	/** The file pattern. */
	private String filePattern;

	public TransferToRepositoryProcess(List<AudioFile> files, int structure, String filePattern) {
		super(files);
		this.structure = structure;
		this.filePattern = filePattern;
	}

	@Override
	public String getProgressDialogTitle() {
		return LanguageTool.getString("COPYING_TO_REPOSITORY");
	}
	
	@Override
	protected String getDestination() {
		return RepositoryHandler.getInstance().getRepositoryPath();
	}
	
	@Override
	protected File transferAudioFile(File destination, AudioFile file) throws IOException {
		File destDir = ExportFilesProcess.getDirectory(file, destination, structure, false);
		FileUtils.copyFileToDirectory(file.getFile(), destDir);
		File destFile = new File(StringUtils.getString(destDir.getAbsolutePath(), SystemProperties.fileSeparator, file.getFile().getName()));
		if (filePattern != null) {
			String newName = FileNameUtils.getNewName(filePattern, file);
			File fileRenamed = new File(StringUtils.getString(destDir.getAbsolutePath(), SystemProperties.fileSeparator, newName));
			destFile.renameTo(fileRenamed);
		}
		return destFile;
	}
	
}
