/*
 * aTunes 1.12.0
 * Copyright (C) 2006-2009 Alex Aranda, Sylvain Gaudard, Thomas Beckers and contributors
 *
 * See http://www.atunes.org/wiki/index.php?title=Contributing for information about contributors
 *
 * http://www.atunes.org
 * http://sourceforge.net/projects/atunes
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

package net.sourceforge.atunes.kernel.modules.player.mplayer;

import net.sourceforge.atunes.kernel.handlers.VisualHandler;
import net.sourceforge.atunes.kernel.modules.podcast.PodcastFeedEntry;

/**
 * The Class PodcastFeedEntryMPlayerOutputReader.
 */
class PodcastFeedEntryMPlayerOutputReader extends MPlayerOutputReader {

	/** The podcast feed entry. */
	private PodcastFeedEntry podcastFeedEntry;

	/**
	 * Instantiates a new podcast feed entry m player output reader.
	 * 
	 * @param handler
	 *            the handler
	 * @param process
	 *            the process
	 * @param podcastFeedEntry
	 *            the podcast feed entry
	 */
	PodcastFeedEntryMPlayerOutputReader(MPlayerHandler handler, Process process, PodcastFeedEntry podcastFeedEntry) {
		super(handler, process);
		this.podcastFeedEntry = podcastFeedEntry;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * net.sourceforge.atunes.kernel.modules.mplayer.MPlayerOutputReader#init()
	 */
	@Override
	protected void init() {
		super.init();

		handler.setCurrentLength(podcastFeedEntry.getDuration() * 1000);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * net.sourceforge.atunes.kernel.modules.mplayer.MPlayerOutputReader#read
	 * (java.lang.String)
	 */
	@Override
	protected void read(String line) {
		super.read(line);

		// When starting playback, update status bar
		if (line.startsWith("Starting playback")) {
			VisualHandler.getInstance().updateStatusBar(podcastFeedEntry);
			if (!started) {
				handler.notifyRadioOrPodcastFeedEntry();
				started = true;
			}
		}
	}

}
